;Originally started out as a port of William White's Ti-86 memorize pi/e program,
;but built from mostly scratch now, with the DAT tables being the only real remaining 
;artifacts of the 86 code.
;Mem pi/e for the 83+ v1.2
;by Boxknife
;contact: goodbitster at gmail dot com


; Recoded slightly by William White, for the TI-73's Mallard shell.
.nolist
#include "mallard.inc"
.list

.org userMem
.db $D9,$00,"Duck"
.dw Start
.db "Memorize PI or E",0

Start:
    ;program code begins here
 
  b_call(_PushRealO1) ;Push Program's name onto the stack - it's in OP1 when the program starts.
                      ;This is for program writeback later on.
  
  res indicRun, (iy+indicFlags) ;Turn off the little running bar at the top
  
  ld c, 0 ; Make sure bc is empty.
  ld b, 0
  

TitleScreen: 
  
  ;Displays Title Screen

  res AppAutoScroll, (iy+AppFlags)
  b_call(_ClrScrnFull)
  set TextInverse, (iy+TextFlags) 
  ld hl, title
  ld a, 0
  ld (curCol), a
  ld (curRow), a
  b_call(_PutS)
  res TextInverse, (iy+TextFlags)
  
  ld (curCol), a
  inc a
  ld (curRow), a
  ld hl, infoline
  b_call(_PutS)
  b_call(_newline)
  ld a, 196
  b_call(_PutC)
  ld hl, lchsmsg
  b_call(_PutS)
  ld hl, (highscorepi)
  b_call(_disphl)
  b_call(_newline)
  ld a, 219
  b_call(_PutC)
  ld hl, lchsmsg
  b_call(_PutS)
  ld hl, (highscoree)
  b_call(_disphl)
  b_call(_newline)
  b_call(_newline)
  ld hl, portmsg
  b_call(_puts)



  set TextInverse, (iy+TextFlags)
  ld hl, titlebot
  ld a, 7
  ld (curRow), a
  ld a, 0
  ld (curCol), a
  b_call(_PutS)
  res TextInverse, (iy+TextFlags)
  set AppAutoScroll, (iy+AppFlags)
  
tloop:      ;Wait until enter or clear key is pressed
  b_call(_getKey)
  cp kEnter
  jp z, LoadPI
  cp kLeft
  jp z, Loade
  cp kRight
  jp z, LoadPi
  cp kClear
  jp z, Quit
  cp kDown
  jp z, Downpage
  jr tloop


ShowAbout:

  ;Shows "about" pages. Scrolling capable. Must set MAXPAGE define at the top to total number of pages.
  ;Each page must contain exactly 96 characters, and only the final page is terminated by a null.

  b_call(_ClrScrnFull) 
ShowAboutNoClr:
 ld a, (curpage)
  cp 0
  jp z, TitleScreen
  
  res AppAutoScroll, (iy+AppFlags)
  ld hl, titlebar
  ld a, 0
  ld (curCol), a
  ld (curRow), a
  set TextInverse, (iy+TextFlags) 
  b_call(_PutS)
  res TextInverse, (iy+TextFlags) 

  ld a, (CurPage) 
  ld b, a
  ld de, aboutmsg ;Puts the aboutmsg into de, then flips it with hl
  ex de, hl
  cp 1
  jr z, SkipSet
  dec b



  ld de, 96  ;Calculates page number 
SetPage:
  add hl, de ;Adds 96 to the pointer for each page
  djnz SetPage

SkipSet:
  ld a, 0
  ld (curCol), a
  inc a
  ld (curRow), a
  b_call(_PutS)
  
  set TextInverse, (iy+TextFlags) 
  ld a, 0
  ld (curCol), a
  ld a, 7
  ld (curRow), a
  
  
  ld hl, lastbar
  ld a, (curpage)
  cp MAXPAGE
  jr z, skipmid
  ld hl, bottombar
skipmid:
  b_call(_PutS)
   res TextInverse, (iy+TextFlags) 
   set AppAutoScroll, (iy+AppFlags)
  
sloop:      ;Wait until enter or clear key is pressed
  b_call(_getKey)
  cp kClear
  jp z, Quit
  cp kUp
  jp z, UpPage
  cp kDown
  jp z, DownPage
  cp kRight
  jp z, LoadPi
  cp kLeft
  jp z, Loade
  cp kEnter
  jp z, LoadPi
  
  jr sloop  
UpPage:
  ld a, (curpage)
  cp 0
  dec a
  ld (curpage), a
  jp ShowAbout

DownPage:  
  ld a, (curpage)
  cp MAXPAGE
  jp z, ShowAboutNoClr
  
  inc a
  ld (curpage), a
  jp ShowAbout
  

LoadSel:
    bit 1, (iy+Asm_Flag1)
    jr z, LoadPi
    
Loade:
    set 1, (iy+Asm_Flag1)
    b_call(_ClrScrnFull)
    ld a, 0
    ld (curRow), a
    ld (curCol), a
    ld a, 219
    b_call(_PutC)
    call PutEq
    ld ix, eDat
    ld a, 0
    ld b, a
    ld c, a
    jr TestLoop
  

  
LoadPI: ;Loads Pi and initializes the counter.
  res 1, (iy+Asm_Flag1)
  b_call(_ClrScrnFull)
  ld a, 0
  ld (curRow), a
  ld (curCol), a
  ld a, 196 ;ASCII value of pi character.
  b_call(_PutC)
  call PutEq
  ld ix, piDat ;makes ix point to the address of piDat.
  
  ld b, 0
  ld c, 0
  
  
TestLoop:
    ld a, (ix) ;Put the value of pi at a?
    
    or a      ;These two lines check to see if
    jp z, Win ;You've memorized it to all the digits here.
  
    b_call(_GetKey) ;GetKey destroys the DE and HL registers.
  
      or a
    
      jr z, TestLoop ;If there's nothing from getkey, just start TestLoop over from here
    
      cp kClear      ;If the player presses the clear button, exit.
      jr z, Quit
      
      cp kEnter
      jp z, Lose
    
      cp kDecPnt      ;Throw out anything if it's not in the numpad or the Clear key.
      jr c, TestLoop
      cp k9+1
      jr nc, TestLoop
    
    
    
    sub 94 ;Subtract 94 from the scankey value returned from GetKey to convert it to ASCII code for the number
    
    cp (ix) ; Compare the value of the key pressed to the value of PI at IX
    
      jr nz, Lose ; Player entered the wrong value
    
    inc ix
    
    push af
    ld a, (CurRow)
    cp 6
    call z, Last
    pop af
    
    
    cp '/'    ;If it's a /, then replace it with '.' and print it.
    push af
    call z, InsPeriod
    b_call(_PutC)
    pop af
    cp '/'
    jr z, TestLoop
    inc bc  ;Increment current score

    
 jr TestLoop
 
 

HighSel:
    bit 1, (iy+Asm_Flag1)
    jr z, PISaveS
 
 
;*****************************************************************
;*****************************************************************  
eSaveS:
  ;Saves high score using by-the-book program writeback
  ld a, (bc) ;Returns if bc is 0 to prevent the destruction of the high score
  cp 0
  ret z
  
  ld (highscoree), bc

  b_call(_PopRealO1)     ;Retrieve OP1 for writeback
  b_call(_ChkFindSym)
  ld hl, eDataStart - $9D95 + 4 ;hl=distance in bytes between highscore and program start
  add hl, de ;Hl now points to the data location
  ex de, hl ;hl is now the value, de is now the data location
  ld hl, eDataStart
  ld bc, eDataEnd - eDataStart
  ldir
  ret

  

;*****************************************************************
;*****************************************************************  


PISaveS:
  ;Saves high score using by-the-book program writeback
  ld a, (bc) ;Returns if bc is 0 to prevent the destruction of the high score
  cp 0
  ret z
  
  ld (highscorepi), bc

  b_call(_PopRealO1)     ;Retrieve OP1 for writeback
  b_call(_ChkFindSym)
  ld hl, PiDataStart - $9D95 + 4 ;hl=distance in bytes between highscore and program start
  add hl, de ;Hl now points to the data location
  ex de, hl ;hl is now the value, de is now the data location
  ld hl, PiDataStart
  ld bc, PiDataEnd - PiDataStart
  ldir
  ret
  

;*****************************************************************
;*****************************************************************  

Quit:
  ;Resets cursor, clears the screen, then quits.
      xor a ; Clear flags
  call PutHSSel
  sbc hl, bc
  call c, HighSel

  b_call(_ClrScrnFull)
  ld a, 0
  ld (curRow), a
  ld (curCol), a
 ; ld h, 0
  ;ld l, 0
 ; ld hl, (highscore)
  ;b_call(_DispHL)
  ret

;*****************************************************************
;*****************************************************************      
    
Lose:
  set TextInverse, (iy+TextFlags) ;Highlight text
  
  push af
    ld a, (curRow)
    cp 6
    call z, Last
  pop af
  
  cp '/'
  call z, InsPeriod
    b_call(_PutMap)
    
  res TextInverse, (iy+TextFlags) ;Turn off highlighting
 
  b_call(_newline)
  b_call(_newline) ;Skips over to a new line and prints "Digits ="
  ld hl, Ans
  b_call(_PutS)
  call PutEq
  
  ld a, (ix)      ;Prints out four correct digits
   cp '/'   
    call z, InsPeriod
  b_call(_PutC)
  inc ix
  ld a, (ix)
     cp '/'   
    call z, InsPeriod
  b_call(_PutC)
  inc ix
  ld a, (ix)
     cp '/'   
    call z, InsPeriod
  b_call(_PutC)
  inc ix
  ld a, (ix)
     cp '/'   
    call z, InsPeriod
  b_call(_PutC)
  
  b_call(_newline)
  ld hl, correct
  b_call(_PutS)
  ld l, c
  ld h, b
  push bc
   b_call(_DispHL)
 
  xor a   ;Check highscore
  ;ld hl, (highscore)
  call PutHSSel
  sbc hl, bc   
  call c, HighSel ;If the current score is greater than the last high score, write it to the high score variable.
  
  b_call(_newline)
  ld hl, hsmsg
  b_call(_PutS)
  call PutHSSel
  b_call(_DispHL)
  b_call(_newline)
    ld hl, entmsg
  ld a, 0
  ld (curCol), a
  ld a, 7
  ld (curRow), a
  res AppAutoScroll, (iy+AppFlags)
  set TextInverse, (iy+TextFlags) 
  b_call(_PutS)
  res TextInverse, (iy+TextFlags) 
  set AppAutoScroll, (iy+AppFlags)
  
  pop bc


Loop1:      ;Wait until enter or clear key is pressed
  b_call(_getKey)
  cp kRight
  jp z, LoadSel
  cp kEnter
  jp z, LoadSel
  cp kClear
  jp z, Quit
  jr Loop1
  
PutHSSel:
    bit 1, (iy+Asm_Flag1)
    ld hl, (highscorepi)
    ret z
    ld hl, (highscoree)
    ret

;*****************************************************************
;*****************************************************************    
Win:

  ;Prints "Winner" and quits. 
  ;You're a "Winner" if you memorize pi to all 1000 places of the program
  
  call Clr
  ld hl, Wt
  b_call(_PutS)
  ret

;*****************************************************************
;*****************************************************************
Last: 

;Clears the screen if the column and row are at the end.

  ld a, (curCol)
  cp 15
  call z, Clr
  ret

;*****************************************************************
;******************************************************************    
InsPeriod:

;Inserts a period into a for the following PutC

 ld a,'.'
 ret    
    
;*****************************************************************    
;*******************************************************************    
PutEq: 

;Prints an equal sign.

 ld hl, Equal
 b_call(_PutS)
 ret
    
;*****************************************************************    
;*******************************************************************
Clr:

;Clears the screen while saving the values of the registers
    push af
    push bc
    push de
    push hl
    push ix
    b_call(_ClrScrnFull)
    ld a, 0
    ld (curCol), a
    ld (curRow), a
    pop ix
    pop hl
    pop de
    pop bc
    pop af
    ret
;*******************************************************************
;*****************************************************************

PiDataStart: 
highscorepi: .dw $0
PiDataEnd:
eDataStart:
highscoree: .dw $0
eDataEnd:
Wt: .db "Winner!",0
Ans: .db "Next digits",0
Equal: .db "=",0
entmsg: .db "   Press Enter  ",0
titlebot: .db 207,219,"   Start    ",196,5, 0
title: .db "  Mem ",196,209,219," v1.2   ", 0
infoline: .db 193,"Clear] quits.", 0
correct: .db "# Correct:", 0
portmsg: .db "Press ",240," for moreinformation.",0
hsmsg: .db "High Score:", 0
lchsmsg: .db " score:", 0
curpage: .db 0

titlebar: 
.db "*About*   Start",5, 0
  
bottombar:
.db 239,"Back      More", 240, 0
lastbar
.db 239,"Back           ", 0
  
aboutmsg:
.db "Starting:       Select pi or e  by pressing the right or left   arrow keys,     respectively.   "
.db "Play:           Input digits of your chosen     number. An errorwill reset      the digits,     "
.db "highlight your  error, and printthe next four   digits starting from the digit  of your mistake."
.db "TI-83+ code by  Boxknife        Remade for the  73 by Will White                                "
.db "You can contact Will White at:  seklorean @     yahoo.com                                       ", 0

 



piDAT:
.db "3/141592653589793238462643383279502884197169399375105820974944592307816406286208"
.db "99862803482534211706798214808651328230664709384460955058223172535940812848111745"
.db "02841027019385211055596446229489549303819644288109756659334461284756482337867831"
.db "65271201909145648566923460348610454326648213393607260249141273724587006606315588"
.db "17488152092096282925409171536436789259036001133053054882046652138414695194151160"
.db "94330572703657595919530921861173819326117931051185480744623799627495673518857527"
.db "24891227938183011949129833673362440656643086021394946395224737190702179860943702"
.db "77053921717629317675238467481846766940513200056812714526356082778577134275778960"
.db "91736371787214684409012249534301465495853710507922796892589235420199561121290219"
.db "60864034418159813629774771309960518707211349999998372978049951059731732816096318"
.db "59502445945534690830264252230825334468503526193118817101000313783875288658753320"
.db "83814206171776691473035982534904287554687311595628638823537875937519577818577805"
.db "32171226806613001927876611195909216420198938095257201065485863278865936153381827"
.db "96823030195203530185296899577362259941389124972177528347913151557485724245415069"
.db "59508295331168617278558890750983817546374649393192550604009277016711390098488240"
.db "12858361603563707660104710181942955596198946767837449448255379774726847104047534"
.db "64620804668425906949129331367702898915210475216205696602405803815019351125338243"
.db "00355876402474964732639141992726042699227967823547816360093417216412199245863150"
.db "30286182974555706749838505494588586926995690927210797509302955321165344987202755"
.db "96023648066549911988183479775356636980742654252786255181841757467289097777279380"
.db "00816470600161452491921732172147723501414419735685481613611573525521334757418494"
.db "68438523323907394143334547762416862518983569485562099219222184272550254256887671"
.db "79049460165346680498862723279178608578438382796797668145410095388378636095068006"
.db "42251252051173929848960841284886269456042419652850222106611863067442786220391949"
.db "45047123713786960956364371917287467764657573962413890865832645995813390478027590",0


eDAT:
.db "2/718281828459045235360287471352662497757247093699959574966967627724076630353547"
.db "59457138217852516642742746639193200305992181741359662904357290033429526059563073"
.db "81323286279434907632338298807531952510190115738341879307021540891499348841675092"
.db "44761460668082264800168477411853742345442437107539077744992069551702761838606261"
.db "33138458300075204493382656029760673711320070932870912744374704723069697720931014"
.db "16928368190255151086574637721112523897844250569536967707854499699679468644549059"
.db "87931636889230098793127736178215424999229576351482208269895193668033182528869398"
.db "49646510582093923982948879332036250944311730123819706841614039701983767932068328"
.db "23764648042953118023287825098194558153017567173613320698112509961818815930416903"
.db "51598888519345807273866738589422879228499892086805825749279610484198444363463244"
.db "96848756023362482704197862320900216099023530436994184914631409343173814364054625"
.db "31520961836908887070167683964243781405927145635490613031072085103837505101157477"
.db "04171898610687396965521267154688957035035402123407849819334321068170121005627880"
.db "23519303322474501585390473041995777709350366041699732972508868769664035557071622"
.db "68447162560798826517871341951246652010305921236677194325278675398558944896970964"
.db "09754591856956380236370162112047742722836489613422516445078182442352948636372141"
.db "74023889344124796357437026375529444833799801612549227850925778256209262264832627"
.db "79333865664816277251640191059004916449982893150566047258027786318641551956532442"
.db "58698294695930801915298721172556347546396447910145904090586298496791287406870504"
.db "89585867174798546677575732056812884592054133405392200011378630094556068816674001"
.db "69842055804033637953764520304024322566135278369511778838638744396625322498506549"
.db "95886234281899707733276171783928034946501434558897071942586398772754710962953741"
.db "52111513683506275260232648472870392076431005958411661205452970302364725492966693"
.db "81151373227536450988890313602057248176585118063036442812314965507047510254465011"
.db "72721155519486685080036853228183152196003735625279449515828418829478761085263981",0



.end